# set working directory to the folder with all of the data
setwd("~/PerformancePersistenceCode")

# import data type of each fund
fundtypes<-read.table("fundtypes.csv", header=TRUE, sep=",", stringsAsFactors = FALSE)
fundtypes<-fundtypes[order(fundtypes$crsp_fundno, fundtypes$caldt),]
fundtypes$year<-as.numeric(substr(fundtypes$caldt,1,4))

# let the percent held in equities be the sum of percentage held in common and preferred stocks
fundtypes$per_pref[is.na(fundtypes$per_pref)]<-0
fundtypes$per_com[is.na(fundtypes$per_com)]<-0
fundtypes$per_com<-fundtypes$per_com+fundtypes$per_pref
fundtypes<-fundtypes[,colnames(fundtypes) %in% c("crsp_fundno","year","per_com","index_fund_flag","et_flag","crsp_obj_cd","wbrger_obj_cd")]

# backfill blank rows in data (assume that a fund kept the same type for blank years)
blank<-fundtypes[which(fundtypes$crsp_obj_cd==""),]
fundtypes<-fundtypes[which(fundtypes$crsp_obj_cd!=""),]
for (i in 1:nrow(blank)){
  crsp<-blank$crsp_fundno[i] # find crsp_fundno we are interested in
  year<-blank$year[i]
  temp<-fundtypes[which(fundtypes$crsp_fundno==crsp),] # get all that are same crsp_fundno
  if (nrow(temp)>0){
    dist<-abs(temp$year-year)  # find minimum years difference among all with same crsp_fundno
    blank$index_fund_flag[i]<-temp$index_fund_flag[min(which(dist==min(dist)))]
    blank$et_flag[i]<-temp$et_flag[min(which(dist==min(dist)))]
    blank$crsp_obj_cd[i]<-temp$crsp_obj_cd[min(which(dist==min(dist)))]
    blank$wbrger_obj_cd[i]<-temp$wbrger_obj_cd[min(which(dist==min(dist)))]
    blank$per_com[i]<-temp$per_com[min(which(dist==min(dist)))]
  }
}
fundtypes<-rbind(fundtypes, blank)
fundtypes<-fundtypes[order(fundtypes$crsp_fundno, fundtypes$year),]

# add in last year (assume fund kept the same type its final year in sample)
lastyear<-fundtypes[!duplicated(fundtypes$crsp_fundno, fromLast=T),]
lastyear$year<-as.character(as.numeric(lastyear$year)+1)
fundtypes<-rbind(fundtypes, lastyear)
fundtypes<-fundtypes[order(fundtypes$crsp_fundno, fundtypes$year),]

# add in formation year (assume fund had the same type its first year in sample)
firstyear<-fundtypes[!duplicated(fundtypes$crsp_fundno),]
firstyear$year<-as.character(as.numeric(firstyear$year)-1)
fundtypes<-rbind(fundtypes, firstyear)
fundtypes<-fundtypes[order(fundtypes$crsp_fundno, fundtypes$year),]

# restrict to equity domestic, excluding sector, hedged, and short funds
crspkeep<-c("EDCL","EDCM","EDCS","EDCI","EDYG","EDYB","EDYI")
# wiesenberger codes: G, G-I, AGG, GCI, GRI, GRO, LTG, MCG, SCG will be kept, if crsp_obj_cd is "M" and per_com between 80 and 105 as in kacperczyk, sialm, zheng
wbrgrkeep<-c("G","G-I","AGG","GCI","GRI","GRO","LTG","MCG","SCG")
fundtypes<-fundtypes[which(fundtypes$crsp_obj_cd %in% crspkeep | (fundtypes$crsp_obj_cd=="M" & fundtypes$wbrger_obj_cd %in% wbrgrkeep & fundtypes$per_com>=80 & fundtypes$per_com<=105)),]
# exclude pure index funds
fundtypes<-fundtypes[which(fundtypes$index_fund_flag!="D"),]
# exclude ETFs and ETNs
fundtypes<-fundtypes[which(fundtypes$et_flag!="F" & fundtypes$et_flag!="N"),]


# make list of kept fund-years to export
keepfunds<-fundtypes[,colnames(fundtypes) %in% c("crsp_fundno","year")]

# export keepfunds to keepfunds.csv
write.csv(keepfunds, row.names=FALSE, file="keepfunds.csv")